
# q-expansion of newform 7569.2.a.y, downloaded from the LMFDB on 18 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7569
weight = 2
poly_data = [9, -27, 13, 15, -8, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[3, -5, -1, 1, 0, 0], 1], [[-12, 16, 12, -8, -2, 1], 3], [[-24, 16, 15, -8, -2, 1], 3], [[-13, 19, 7, -9, -1, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0], [1, -1, 0, 0, 0, 1], [-1, 0, 1, 1, 0, 0], [-2, 0, -1, 0, 0, 1], [-1, 1, 1, -1, 0, -1], [-1, 0, -1, 0, 0, -1], [-2, 2, 0, 1, 1, 0], [1, 1, 1, 0, 0, 1], [0, 0, 0, 0, 0, 0], [2, 0, 1, 2, -1, 1], [-2, -1, 1, 1, -2, 0], [-4, 0, 1, -2, -1, 0], [-1, 2, 2, -1, -1, -2], [0, 2, 0, -3, 0, 0], [3, -1, -2, -1, -2, 2], [2, 2, -1, -1, 1, -2], [1, -2, -4, -2, 0, 2], [-3, 1, 1, 3, -1, 1], [-1, 1, 1, 0, -3, -1], [1, -2, 1, 1, 1, -3], [3, 0, -4, -6, 2, -2], [6, -1, -2, -3, 3, 0], [-3, 4, 0, 2, 1, 2], [-3, 1, -2, 1, 3, -2], [-9, -1, 1, 2, -2, -1], [6, 1, 0, 3, 1, 3], [-2, 5, 1, 4, -1, -1], [-2, -3, -2, 5, 2, -3], [8, 0, 0, -8, -1, 0], [4, 3, 0, -1, 2, -3], [-6, -1, 1, 3, 0, -3], [0, -2, -3, -4, 4, -4], [-1, 5, -1, -5, 0, -3], [6, -2, -3, -6, -3, 0], [10, -1, -4, -5, -2, 0], [2, -2, 1, -6, 2, 3], [-11, 1, -1, 4, 0, -4], [6, -4, 0, -6, 3, -3], [2, 1, 0, 7, 5, 0], [3, -4, 3, -2, -1, -5], [7, -3, 3, -6, -1, 2], [0, -2, 0, -2, 2, 4], [-12, 4, -3, 9, -5, 0], [1, 1, -2, -6, 0, -2], [-2, -3, -6, 7, 1, 3], [-1, 4, 6, 6, 0, -3], [-3, 5, 1, 0, -5, 6], [1, 3, 3, 11, 1, 3], [3, 4, 4, 3, -3, 1], [1, 0, 3, 10, -4, -1], [-4, -4, -2, -7, 1, 1], [-6, -4, -1, 4, 3, -5], [-15, 0, 4, -1, 1, -1], [-1, -4, 0, 13, -4, 3], [-9, -4, -2, 3, 0, 0], [5, -5, 0, 4, 5, 3], [6, 2, 6, 3, -3, 4], [-1, 1, -1, -4, -6, 2], [2, -5, -4, -5, 2, -6], [9, -1, -3, -9, 2, 7], [2, -4, -3, -5, 8, 0], [-4, -7, 2, 5, 3, 2], [-16, 6, -1, 3, -6, -4], [-8, 6, 1, -4, 5, 0], [3, -2, 4, 10, -1, 4], [-9, 6, 6, 3, 1, 0], [4, -1, -1, -12, 3, 5], [-16, -6, -1, 1, 2, 3], [9, 8, 5, 0, -1, -5], [-8, 6, 5, 0, -6, 4], [-5, 5, 9, 8, -2, 3], [9, -6, -2, -12, 4, 3], [5, -4, 4, 6, -5, -1], [6, -4, -9, -8, 5, -1], [-9, -6, -9, -1, 1, 2], [-6, 0, -4, -7, 4, -1], [14, -4, -3, -5, 0, -6], [15, -2, 3, -12, 6, -6], [4, 5, 2, -3, 5, -7], [1, -5, 5, 1, 5, -4], [-9, -1, 3, 1, -8, 2], [-13, 1, -2, 11, -5, 7], [-2, -5, -8, -2, -5, 0], [-14, 0, 1, 3, 2, -7], [5, -5, -3, -2, 5, 0], [3, -3, 3, 1, 2, 4], [-11, 4, -3, -3, 1, 1], [1, 9, 11, -5, 2, -7], [6, -8, -6, -4, 3, 6], [-7, 3, -3, 5, -2, 4], [-16, 1, -1, -3, 3, -1], [1, 10, 5, 11, 3, -5], [-25, 6, -5, 3, 6, -7], [-5, -5, -2, -5, 0, 2], [-9, 4, 11, 4, -1, -2], [11, -5, -4, 3, 0, -1], [-3, 5, -8, 6, -3, 0], [-6, -3, -3, 10, 1, -1], [-7, 5, -1, -1, 1, 5], [-18, -2, 1, 0, -4, -2], [17, -6, 6, -13, 1, -2], [-1, 3, 6, 8, 4, 1], [-22, 4, 4, 9, 3, -10], [1, 6, -5, 0, -7, -1], [3, -2, 6, 2, -1, 2], [-1, 2, 3, -4, 4, 4], [-1, 9, -4, 2, 1, -5], [-26, 1, -3, 4, 2, -7], [-5, -7, 5, 6, 10, -5], [-12, 4, -7, 7, -4, -4], [9, -3, -4, -1, -1, -1], [-2, 0, -5, 4, -1, -10], [8, 4, -5, -9, 8, 3], [6, -3, -3, -6, 3, -8], [9, 1, 4, -6, -6, 3], [-25, 10, -7, 6, -6, -9], [-5, -7, 8, 15, -3, 4], [6, 8, 11, 5, -2, -4], [-3, -4, -4, 2, 4, -4], [4, -6, 2, 0, -12, 5], [-5, -17, 0, 9, 0, 8], [-8, 7, 4, 5, 7, -6], [3, -3, 8, -10, 4, 2], [-4, -10, 1, -11, 4, 1], [-16, -2, -7, 6, -6, 2], [16, 6, 7, 8, 0, -5], [7, 5, -4, -12, 3, 4], [26, -4, 3, 2, 0, 5], [-23, -1, 2, 23, -1, -3], [-4, -8, -4, -3, -2, 2], [-28, 5, 4, 3, -6, -1], [-4, 2, 5, -10, 2, 4], [-9, -1, -4, -17, -1, -7], [-14, 6, -1, 7, -4, -4], [-12, 8, 2, 2, -4, 2], [2, -7, 0, -1, -5, -2], [-2, -7, -5, -10, 7, -5], [8, -1, -16, -6, -6, -1], [5, 0, -3, -11, -10, 6], [9, -7, -2, -16, -7, 5], [-15, 0, 3, 24, -6, -9], [1, 10, 5, -14, -2, 6], [-4, 0, -1, 26, 1, 1], [5, 6, 15, 3, -8, 2], [-11, 6, -3, -8, -4, 2], [-8, 6, -4, 3, -6, 2], [9, 9, -1, -6, -3, -6], [-7, -9, 6, 5, 0, -7], [10, 0, -13, -3, 0, -2], [3, -2, -5, -12, 4, -15], [14, -16, 1, -1, 4, 7], [8, -1, 5, -9, -8, 5], [6, -7, 3, -5, 5, -11], [-14, 0, -9, -4, -3, 7], [-10, -5, -5, -6, -6, -1], [-3, -13, -7, -11, -3, 1], [-22, 3, 5, 18, -3, -7], [-9, 4, 8, 12, 3, -8], [2, -8, 7, -1, -5, 1], [-9, -11, -6, 5, -2, 11], [-16, 14, -2, 1, -1, -3], [7, -5, 7, -10, 3, -5], [-13, -11, -3, 3, 6, -10], [21, -7, -4, 11, 4, -4], [-19, 2, 15, 20, -5, 4], [18, -4, -6, -7, -4, 5], [-1, 5, 2, 7, 12, 0], [20, -9, 9, 1, 4, 13], [13, -2, 5, 3, -6, 13], [-23, -5, -4, 17, 7, -15], [-6, -8, -1, 12, -14, 9], [9, -9, -4, 11, 7, 11], [5, -6, -5, 3, 3, 9], [27, -3, 11, -13, 8, -4], [3, -18, -5, 2, 7, -1], [-9, 4, 10, 12, -12, 4], [2, 10, 6, 24, -3, 2], [4, -10, -8, 0, 11, -10], [24, -1, -3, -10, -7, 8], [-4, -4, -4, -2, 1, 1], [5, -3, 3, -10, 1, 1], [4, 9, -2, 9, 9, -4], [14, -2, 0, -26, -1, 0], [12, 7, 8, -21, 0, -12], [1, -6, -4, -15, -9, -2], [-20, 8, 7, 30, -1, 5], [-14, -4, 3, -5, 6, -1], [11, -7, -5, 5, -11, 8], [14, 7, 0, 8, -8, -2], [7, 6, -3, 2, -6, 4], [11, 3, -4, 19, -7, -6], [-15, 4, 13, 23, -21, 6], [-4, -11, -15, -4, 4, 10], [15, -5, -3, 3, -6, 9], [-6, 12, 13, 0, 3, -6], [4, -5, -9, -6, 7, 4], [-4, -6, -12, -12, 8, 0], [0, -5, -6, -11, -10, 1], [2, 6, 2, 0, 15, -1], [-1, 2, 3, 7, -7, -3], [-12, 1, 8, 33, 0, 4], [-13, 4, -2, -4, 1, 5], [7, -3, -15, -4, -12, 10], [-11, 11, 12, 2, -14, -3], [-5, -12, 7, -10, 10, 6], [11, 5, 8, -7, -7, 13], [-4, 0, 2, -2, -1, -3], [4, 6, 3, 2, -2, 15], [8, -10, -5, 12, -2, -1], [12, -2, 10, 12, -13, 10], [19, -18, 1, 0, 9, -5], [-27, -3, 3, 1, 0, 4], [8, -14, -2, -14, 14, 3], [-13, 9, 3, -17, 11, 6], [-3, 17, -2, 9, 4, -3], [-11, 0, -1, -6, 5, 8], [-2, -4, 12, 7, -7, -10], [11, 13, 0, -19, -8, 13], [1, -7, -9, -18, 3, 10], [6, -6, -6, 10, 4, 8], [6, 14, -5, -12, 2, -8], [8, -20, 5, 11, -8, 13], [10, 5, 6, -7, 6, 1], [2, -2, -8, 11, 4, 4], [3, -10, 0, 2, 10, 6], [-38, 3, -8, 3, 6, 1], [-30, -7, 4, 19, -7, 16], [10, -10, -7, -27, 7, -5], [-12, 3, -13, -13, -5, -7], [21, -10, -12, 3, -1, 4], [-13, -9, -13, 11, 8, -14], [20, 1, -2, 11, 13, -1], [0, -5, 4, -17, -4, -8], [20, 4, -13, -32, 9, -6], [-24, 6, 1, 11, 7, -22], [4, -11, 4, 15, 1, 7], [22, -6, 1, -19, -2, -6], [-23, -10, 12, 0, 3, -8], [10, -16, -11, -18, -3, 10], [-15, 7, -9, 25, -4, 1], [8, 11, 1, 11, -13, 1], [-8, 11, 12, 6, 2, 8], [-9, -8, 19, -3, 2, -8], [-34, 8, -3, -2, -4, -3], [0, 15, 7, -23, -4, 4], [22, 10, -5, -29, 4, 6], [2, 17, 7, -3, 9, -10], [12, -5, -15, 7, 12, 1], [-7, 22, 6, 2, -20, 10], [27, -6, 4, 5, 1, -3], [-24, -6, -18, -2, 2, -2], [-1, -11, 2, 6, 9, -1], [31, -9, 3, -6, 0, -4], [7, -9, -1, -9, 0, -11], [42, -5, -9, -14, 11, 16], [12, 4, 8, 12, -7, 10], [-14, 16, 4, 16, 1, 9], [10, 5, -5, -14, 14, 2], [9, 3, -10, -7, -9, 3], [-32, 1, 8, 18, -8, 10], [-34, 4, 13, 10, -1, 9], [-7, 4, 23, 16, -9, 6], [30, 8, -12, -11, -11, 8], [22, 11, -5, -22, -8, 12], [16, 13, 3, -14, -2, 0], [-22, 2, -3, 29, 1, -11], [-33, -6, 0, 2, 1, -10], [16, 9, -1, -34, -9, 1], [-3, -15, -3, 13, -7, 4], [-23, 6, 5, -11, -3, -1], [27, -7, 4, 4, -2, 2], [19, -8, -7, 9, 3, 5], [-19, 9, -11, 11, -1, -2], [4, 0, 1, 22, -6, 5], [-27, 8, 2, 40, 4, -7], [31, 5, -16, -16, 10, 0], [1, -3, -1, 3, 10, -5], [18, -10, 0, -50, 6, 1], [-18, -9, 4, 37, -1, 1], [-46, 2, -4, -12, 0, -4], [-28, 9, 5, 41, -7, 1], [-4, -9, 5, 13, 11, 6], [-14, -3, -14, -10, 9, -8], [-25, -5, -1, 26, 13, -13], [23, 0, -4, -2, 23, -9], [-4, -11, -21, 16, 0, 9], [-14, 14, -5, 20, 1, 6], [-9, -17, -7, -5, 4, -10], [10, -16, -8, 12, 3, 16], [-13, 11, 11, 1, -19, 6], [36, 3, -15, -28, 4, -1], [-3, -2, -17, -12, 3, -6], [3, -12, -3, -31, 7, 11], [5, 1, -5, 4, -6, -3], [-36, 8, 3, 19, -22, 13], [-19, 7, 21, 7, 10, -2], [0, 2, -15, 6, 0, -3], [-15, 10, -2, 18, -12, 9], [-2, 12, 10, -24, -5, 1], [8, -13, -14, -17, -2, 1], [13, 23, 5, -1, -8, -3], [-9, 11, 17, 5, -9, -9], [38, -9, 5, -23, 8, -15], [6, -16, -21, -32, 2, -10], [10, -6, -5, 7, -2, 6], [14, 4, 7, -34, -2, -8], [12, 11, 21, 14, -10, 2], [12, -7, -5, 9, 0, 12], [-3, 1, -9, 19, -11, -1], [0, 11, 4, -13, -11, -4], [-25, 4, -3, 5, -11, 4], [3, 3, 15, 1, -1, 16], [-5, 7, -8, 23, -13, -9], [13, 15, 1, -15, -12, 19], [-25, 8, 6, 28, 3, -3], [-18, 15, 11, -9, -3, 3], [-13, -2, 4, 34, 5, -9], [-3, 18, -2, -20, -7, -4], [-13, 6, 9, 12, -21, 8], [8, 15, 19, -25, -6, -7], [10, -2, 1, -16, 0, 1], [-25, 8, -1, 12, 0, -16], [-14, -9, -3, -2, 1, -9], [-6, 12, -1, -17, -1, 16], [3, -3, -2, 9, 3, 4], [48, -9, -3, -11, 13, -4], [16, -16, -8, -21, 4, 25], [-34, -9, 7, 5, -2, -8], [-42, -1, 2, 22, -7, 19], [-27, -3, 16, 27, 1, -12], [-9, 0, -5, 3, 12, -9], [-18, -1, 4, 28, -20, -7], [6, 16, 10, -6, 0, -15], [3, -7, -20, -17, -17, 2], [28, 1, -6, -37, 1, -15], [-34, 12, -5, 29, 2, 10], [15, -17, -25, 21, -3, 3], [45, -2, 3, -32, -2, 2], [15, -1, -6, -1, 17, -1], [-9, 3, 23, 7, -10, -10], [-2, 18, 18, 25, -1, 20], [29, -2, -14, -25, 13, -8], [23, -6, 5, 7, 7, 25], [40, 2, 2, -11, 5, -4], [-14, 12, 8, 4, 14, -19], [-39, 5, -8, 8, 0, 0], [-3, 0, 5, -25, 6, 3], [-24, 1, -7, 24, -21, 12], [-38, -1, 17, 23, 1, -6], [4, 11, 10, -8, -9, -4], [-4, -11, -15, 1, -3, 21], [10, 17, 30, 5, -17, 3], [-14, 5, 12, 9, -7, 20], [-17, -11, -4, 16, -15, 11], [-48, 8, -1, 7, -25, 10], [-5, 0, -2, -41, 11, 17], [-25, 8, -6, 8, 7, -5], [3, -22, -16, 2, 13, -4], [24, 9, 6, 19, 2, 16], [-24, -8, -11, 14, -11, 3], [0, 11, 11, 19, -10, 7], [-8, -8, -8, -19, 1, -9], [11, -1, -1, -10, 16, 10], [-29, -18, -10, -15, 7, 10], [22, -8, 2, -2, -9, -13], [9, -16, -10, 15, 9, -6], [21, -14, 11, -28, 18, 9], [29, 17, 13, 16, -6, -6], [8, 11, 10, 20, -2, 13], [-42, 16, 5, 4, 8, -7], [-11, 13, -10, -22, 10, -6], [-20, -12, -1, 16, -13, 8], [-47, 8, -1, 23, 9, -20], [8, 1, 6, -20, -9, 3], [-18, 10, -2, 16, -19, 4], [20, -9, 13, -32, 14, 6], [-26, 2, -1, 10, 4, 3], [1, 0, 9, 22, 2, -7], [-55, 8, -8, 2, 8, -8], [22, 1, -18, -41, 23, -10], [35, -6, -4, -42, 9, 2], [11, -21, -10, 15, -12, 6], [20, -1, -4, -18, -3, 17], [6, -9, 14, -12, 5, -2], [-19, 30, 9, -12, -9, 14], [14, -24, 6, 10, 3, 15], [10, -7, -16, -25, -7, -9], [-16, 11, -5, 33, -16, 6], [-45, -2, 9, 13, -10, 10], [-19, 6, -2, -20, 1, 10], [32, 7, 4, -38, -7, 9], [44, 1, -19, -17, -7, -6], [12, 2, 12, 26, 0, 0], [-21, 2, -1, -6, -15, 12], [-18, -15, -17, 7, 12, -8], [-10, -12, 5, -6, 18, 0], [-25, 17, 12, -31, 1, 7], [41, 22, -14, -27, -5, 5], [-51, 16, 5, 15, -3, 0], [26, -25, 6, -21, 17, 0], [18, -20, -16, 0, 9, 12], [-9, -3, -23, -16, 15, -18], [9, -30, -6, 10, 14, 7], [-6, 9, -4, -1, 16, -7], [14, 11, -14, -27, 4, -7], [-5, -5, -17, 6, -13, 11], [-30, 20, -8, 2, 7, -1], [-23, 11, 14, 38, -6, 19], [43, -7, 6, -31, 7, 9], [-13, 8, 21, 50, -21, 5], [-46, 25, -7, -1, 13, -11], [9, -7, 10, 14, -20, -4], [-15, 12, 9, 30, 1, -6], [14, 15, -2, -10, -6, -1], [-24, 1, 13, 41, -5, 8], [-3, 7, 0, -10, -5, 2], [-1, -9, 3, -17, -4, 16], [16, -9, -7, -9, 0, -8], [-17, 7, -2, 14, -5, 27], [8, -8, -13, -12, 18, -19], [-4, 15, 2, 27, -7, 0], [23, -3, -3, -31, 19, 7], [-27, 5, -11, 42, -9, 9], [51, -5, -5, -13, 15, -18], [-19, -5, 0, 14, -2, 10], [-17, 22, -1, -14, 11, 2], [49, -13, 7, -14, 5, 5], [20, 10, 2, -19, 12, -4], [10, -5, -14, -39, 24, 4]]
